#!/bin/bash

# Script for automatic changing the broken interfaces
# Vadim Ipatov <euphoria.vi@gmail.com>

# ---------------------------------------------------------------------------- #

LOG_FILE="/var/log/channels.log"
LOG_TTY="/dev/tty8"

CMD_PING1="ping -c 1 ya123.ru"
CMD_PING2="ping -c 1 google123.com"
CMD_SZAP="szap -n 1 -x"

PROV_EARTH="Prostor"
PROV_SAT="Raduga"
PROV_NOCHAN="No channel"

IPT_EARTH_AND_SAT="/etc/iptables/earth_and_sat.conf"
IPT_EARTH="/etc/iptables/earth_only.conf"

EARTH_GW="192.168.1.254"
SAT_GW=""

EARTH=1
SAT=1

CHECK_TIMEOUT=0
RECHECK_TIMEOUT=0

# ---------------------------------------------------------------------------- #

function log () {
    if [ $2=1 ];
    then
        echo "`date +"%D %T"` $1" >> $LOG_FILE
    fi
    
    if [ $3=1 ];
    then
	echo "`date +"%D %T"` $1" >> $LOG_TTY
    fi
}

# ---------------------------------------------------------------------------- #

function change_iface() {
    log "[Message]: Changing interfaces to " 1 1
    
    case "$1" in
	earth_only)
	    log "$PROV_EARTH..." 1 1
	    EARTH=1
	    SAT=0
#	    iptables-restore $IPT_EARTH
#	    route del default
#	    ip route add default $EARTH_GW
	    # ...
	;;
	sat_only)
	    log "$PROV_SAT ..." 1 1
	    EARTH=0
	    SAT=1
	    # ...
	;;
	earth_and_sat)
	    log "$PROV_EARTH + $PROV_SAT..." 1 1
	    EARTH=1
	    SAT=1
#	    iptables-restore $IPT_EARTH_AND_SAT
#	    route del default
	    # ...
	;;
	no_channel)
	    log $PROV_NOCHAN 1 1
	    EARTH=0
	    SAT=0
    esac
}

# ---------------------------------------------------------------------------- #

function test_sat()  {
    $CMD_SZAP | grep "FE_HAS_LOCK" > /dev/null 2>&1;
    
    if [ $? != 0 ];
    then
        log "[Warning]: No signal from satellite." 1 1
	log "           Waiting for $RECHECK_TIMEOUT secs and trying again..." 1 1
        sleep $RECHECK_TIMEOUT

        $CMD_SZAP | grep "FE_HAS_LOCK" > /dev/null 2>&1;
    	if [ $? != 0 ];
        then	
		return 1
        fi
    fi
    
    return 0
}

# ---------------------------------------------------------------------------- #

function test_earth() {
    $CMD_PING1 | grep "100% packet loss" > /dev/null 2>&1;
    
    if [ $? = 1 ];
    then
	log "[Warning]: First test server is not available." 1 1
	log "           Waiting for $RECHECK_TIMEOUT secs and trying again with another..." 1 1
        sleep $RECHECK_TIMEOUT
	
        $CMD_PING2 | grep "100% packet loss" > /dev/null 2>&1;    
	if [ $? = 1 ];
        then
	    log "[Warning]: Second test server is not available too" 1 1
	    return 1
        fi
    fi
    
    return 0
}

# ---------------------------------------------------------------------------- #

while [ true ];
do
# --------------------------------- #
    while [ $EARTH -eq 1 -a $SAT -eq 1 ];
    do
    	log "111111111111111111" 1 1
	test_sat
	if [ $? != 0 ]; 
        then
	    log "[Error]: $PROV_SAT channel is not working!" 1 1
    	    change_iface "earth_only"
	fi
	
        test_earth
	if [ $? != 0 ]; 
        then
	    log "[Error]: $PROV_EARTH channel is not working!" 1 1
	    if [ $SAT -eq 1 ];
	    then
    		change_iface "sat_only"
	    else
		change_iface "no_channel"
	    fi
        fi    

	log "[Message]: Sleeping for $CHECK_TIMEOUT secs before next cheking..." 0 1
        sleep $CHECK_TIMEOUT
    done
# --------------------------------- #    
    while [ $EARTH -eq 1 -a $SAT -eq 0 ];
    do
    	log "2222222222222222" 1 1

	test_sat
	if [ $? = 0 ]; 
        then
	    log "[Message]: Found a signal from $PROV_SAT channel!" 1 1
    	    change_iface "earth_and_sat"
	    break
	fi

        test_earth
	if [ $? != 0 ]; 
        then
	    log "[Error]: $PROV_EARTH channel is not working!" 1 1
    	    change_iface "sat_only"
        fi    

	log "[Message]: Sleeping for $CHECK_TIMEOUT secs before next cheking..." 0 1
	sleep $CHECK_TIMEOUT    
    done
# --------------------------------- #
    if [ $EARTH -eq 0 -a $SAT -eq 1 ];
    then
    	log "3333333333333333333" 1 1
        change_iface "earth_and_sat"

	log "[Message]: Sleeping for $CHECK_TIMEOUT secs before next cheking..." 0 1
	sleep $CHECK_TIMEOUT    
    fi
# --------------------------------- #
    if [ $EARTH -eq 0 -a $SAT -eq 0 ];
    then
    	log "44444444444444444" 1 1
        change_iface "earth_and_sat"

	log "[Message]: Sleeping for $CHECK_TIMEOUT secs before next cheking..." 0 1
	sleep $CHECK_TIMEOUT    
    fi
# --------------------------------- #    
done

exit 0